using System;
using System.Windows.Media;
using System.Windows.Media.Imaging;

namespace MandelbrotDemo
{
    public class BitmapGenerator
    {
        /// <summary>
        /// Create a 8-bit index bitmap of the buffer
        /// </summary>
        /// <param name="imageBuffer"></param>
        /// <param name="width"></param>
        /// <param name="height"></param>
        /// <param name="maxIterations"></param>
        /// <returns></returns>
        public static BitmapSource Create(int[] imageBuffer, int width, int height, int maxIterations, BitmapPalette p)
        {
            var buffer = new byte[width * height];
            for (int i = 0; i < imageBuffer.Length; i++)
            {
                byte b = imageBuffer[i] < maxIterations ? (byte)(Math.Ceiling(255.0 * imageBuffer[i] / maxIterations)) : (byte)0;
                buffer[i] = b;
            }

            int dpi = 120;

            return BitmapSource.Create(width, height, dpi, dpi, PixelFormats.Indexed8, p, buffer, width);
        }
    }

    public static class PaletteHelper {

        public static BitmapPalette CreateGradientPalette(int n1, Color c1, Color c2, Color c3)
        {
            Color[] colors = new Color[256];
            for (int i = 0; i < n1; i++)
                colors[i] = BlendColor(c1, c2, (double)i / n1);
            for (int i = n1; i < 255; i++)
                colors[i] = BlendColor(c2, c3, (double)(i - n1) / (255 - n1));

            colors[0] = Colors.Black;

            return new BitmapPalette(colors);
        }

        /// <summary>
        /// Blend two colors (1-p) of color1 and (p) of color2
        /// </summary>
        /// <param name="color1"></param>
        /// <param name="color2"></param>
        /// <param name="p"></param>
        /// <returns></returns>
        private static Color BlendColor(Color color1, Color color2, double p)
        {
            return Color.FromArgb(255,
                                  (byte)(color1.R * (1 - p) + color2.R * p),
                                  (byte)(color1.G * (1 - p) + color2.G * p),
                                  (byte)(color1.B * (1 - p) + color2.B * p));
        }

        public static BitmapPalette CreateBlusihPalette(byte rotate)
        {
            return CreatePalette(BluishPalette, rotate);
        }

        //            var p = CreateGradientPalette(180, Colors.DarkBlue, Colors.White, Colors.Yellow);
        //            var p = CreateGradientPalette(180, Colors.DarkBlue, Colors.Cyan, Colors.White);

        static readonly byte[,] BluishPalette = {
                                     {0, 0, 7, 100}, 
                                     {40, 32, 107, 203}, 
                                     {107, 237, 255, 255}, 
                                     {163, 255, 170, 0},
                                     {218, 0, 0, 0}
                                 };

        private static BitmapPalette CreatePalette(byte[,] c, byte rotate)
        {
            Color[] colors = new Color[256];
            int b = c.GetUpperBound(0)+1;
            for (int i = 0; i < b; i++)
            {
                int i2 = (i + 1) % b;
                var c1 = Color.FromArgb(255, c[i, 1], c[i, 2], c[i, 3]);
                var c2 = Color.FromArgb(255, c[i2, 1], c[i2, 2], c[i2, 3]);
                int n0 = c[i, 0];
                int n1 = c[i2, 0];
                if (n1 == 0) n1 = 256;
                int n = n1 - n0;
                for (int j = n0; j < n1; j++)
                {
                    int jr = (j + rotate)%255;
                    colors[jr] = BlendColor(c1, c2, (double)(j-n0) / n);
                }
            }
            colors[0] = Colors.Black;
            return new BitmapPalette(colors);
        }

    }
}