﻿using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media.Imaging;
using System.ComponentModel;
using System.Diagnostics;

namespace MandelbrotDemo
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window, INotifyPropertyChanged
    {
        public BitmapSource ImageSource { get; set; }

        #region MaxIterations Dependency Property

        public double MaxIterations
        {
            get { return (double)GetValue(MaxIterationsProperty); }
            set { SetValue(MaxIterationsProperty, value); }
        }

        public static readonly DependencyProperty MaxIterationsProperty =
            DependencyProperty.Register("MaxIterations", typeof(double), typeof(Window1), new UIPropertyMetadata(64.0, MaxIterationsChanged));

        protected static void MaxIterationsChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            ((Window1)d).OnMaxIterationsChanged();
        }

        protected virtual void OnMaxIterationsChanged()
        {
            UpdateFrame();
        }

        #endregion

        #region Rotation dependency property

        public int Rotation
        {
            get { return (int)GetValue(RotationProperty); }
            set { SetValue(RotationProperty, value); }
        }

        public static readonly DependencyProperty RotationProperty =
            DependencyProperty.Register("Rotation", typeof(int), typeof(Window1), new UIPropertyMetadata(-1,RotationChanged));

        private static void RotationChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            ((Window1)d).OnRotationChanged();
        }

        private void OnRotationChanged()
        {
            palette = PaletteHelper.CreateBlusihPalette((byte) Rotation);
            UpdateImage();
        }

        #endregion

        
        #region CalculationTime Dependency Property

        public double CalculationTime
        {
            get { return (double)GetValue(CalculationTimeProperty); }
            set { SetValue(CalculationTimeProperty, value); }
        }

        public static readonly DependencyProperty CalculationTimeProperty =
            DependencyProperty.Register("CalculationTime", typeof(double), typeof(Window1), new UIPropertyMetadata(0.0, CalculationTimeChanged));

        protected static void CalculationTimeChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            ((Window1)d).OnCalculationTimeChanged();
        }

        protected virtual void OnCalculationTimeChanged()
        {
        }

        #endregion
        #region UseParallel Dependency Property

        public bool UseParallel
        {
            get { return (bool)GetValue(UseParallelProperty); }
            set { SetValue(UseParallelProperty, value); }
        }

        public static readonly DependencyProperty UseParallelProperty =
            DependencyProperty.Register("UseParallel", typeof(bool), typeof(Window1), new UIPropertyMetadata(true, UseParallelChanged));

        protected static void UseParallelChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            ((Window1)d).OnUseParallelChanged();
        }

        protected virtual void OnUseParallelChanged()
        {
            UpdateFrame();
        }

        #endregion

        
        IEquationSolver parallelSolver;
        IEquationSolver simpleSolver;
        SolverFrame frame;

        public Window1()
        {
            InitializeComponent();
            DataContext = this;
            Rotation = 0;

            parallelSolver = new ParallelMandelbrotSolver();
            simpleSolver = new MandelbrotSolver();
            frame = new SolverFrame();
        }

        private void UpdateFrame()
        {
            frame.MaxIterations = (int)MaxIterations;
            frame.Width = (int) Grid1.ActualWidth;
            frame.Height = (int) Grid1.ActualHeight;
            frame.Solver = UseParallel ? parallelSolver : simpleSolver;

            Cursor = Cursors.Wait;

            var w = new Stopwatch();
            w.Start();
            frame.Update();
            CalculationTime = w.ElapsedMilliseconds;

            UpdateImage();

            Cursor = Cursors.Arrow;
        }

        private void UpdateImage() {
            if (frame == null)
                return;
        // todo: is this leaking?
            ImageSource = null;
            ImageSource = BitmapGenerator.Create(frame.ImageBuffer, frame.Width, frame.Height, frame.MaxIterations, palette);
            RaisePropertyChanged("ImageSource");
        }

        private BitmapPalette palette;

        #region PropertyChanged Block
        public event PropertyChangedEventHandler PropertyChanged;

        protected void RaisePropertyChanged(string property)
        {
            var handler = PropertyChanged;
            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(property));
            }
        }
        #endregion

        private void Image_MouseWheel(object sender, MouseWheelEventArgs e)
        {
            // zoom the image
            var img = (Image)sender;
            var pos = e.GetPosition(sender as Image);
            frame.Zoom(pos.X / img.ActualWidth, pos.Y / img.ActualHeight, e.Delta * 0.001);
            UpdateFrame();
        }

        Point lastPoint;
        private void Image_MouseDown(object sender, MouseButtonEventArgs e)
        {
            var img = (Image)sender;
            img.CaptureMouse();
            lastPoint = e.GetPosition(img);
        }

        private void Image_MouseMove(object sender, MouseEventArgs e)
        {
            var img = (Image)sender;
            if (img.IsMouseCaptured)
            {
                // pan the image
                var pt = e.GetPosition(img);
                frame.Pan((lastPoint.X - pt.X) / img.ActualWidth, (lastPoint.Y - pt.Y) / img.ActualHeight);
                lastPoint = pt;
                UpdateFrame();
            }

        }

        private void Image_MouseUp(object sender, MouseButtonEventArgs e)
        {
            var img = sender as Image;
            img.ReleaseMouseCapture();
        }

        private void Grid1_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            // Update when the window has been resized
            UpdateFrame();
        }

        protected override void OnKeyDown(KeyEventArgs e)
        {
            base.OnKeyDown(e);

            if (e.Key == Key.A)
                Zoom(0.2);
            if (e.Key == Key.Z)
                Zoom(-0.2);
        }

        private void Zoom(double p)
        {
            frame.Zoom(0.5, 0.5, p);
            UpdateFrame();
        }

    }
}
