﻿#if NET35
// .NET 3.5 - Referencing System.Threading (Parallel extensions)
using System.Threading;
#else
// .NET 4.0
using System.Threading.Tasks;
using System.Collections.Concurrent;
#endif

namespace MandelbrotDemo
{
    /// <summary>
    /// Interface for the solvers
    /// </summary>
    public interface IEquationSolver
    {
        int[] Solve(double[] x, double[] y, int maxit);
    }

    /// <summary>
    /// Solving the equation in a non-parallel loop
    /// </summary>
    public class MandelbrotSolver : IEquationSolver
    {
        public int[] Solve(double[] x, double[] y, int maxit)
        {
            int n = x.Length;
            var result = new int[n];
            for (int i = 0; i < n; i++)
                result[i] = MandelbrotEquation.Solve(x[i], y[i], maxit);
            return result;
        }

    }

    public class ParallelMandelbrotSolver : IEquationSolver
    {
        public int[] Solve(double[] x, double[] y, int maxit)
        {
            int n = x.Length;
            var result = new int[n];
            Parallel.For(0, n, i => result[i] = MandelbrotEquation.Solve(x[i], y[i], maxit));
            return result;
        }
    }

    public class Parallel2MandelbrotSolver : IEquationSolver
    {
        public int[] Solve(double[] x, double[] y, int maxit)
        {
            int n = x.Length;
            var result = new int[n];
            // .NET 4.0
            // var rangePartitioner = Partitioner<int>.Create(0, source.Length);
            Parallel.For(0, n, i => result[i] = MandelbrotEquation.Solve(x[i], y[i], maxit));
            return result;
        }
    }

    public class Parallel3MandelbrotSolver : IEquationSolver
    {
        int parts = 8;
        public int[] Solve(double[] x, double[] y, int maxit)
        {
            int n = x.Length;
            var result = new int[n];
            int[] l = new int[n];
            for (int i = 0; i < parts - 1; i++)
                l[i] = i * n / parts;
            l[parts - 1] = n;

            Parallel.For(0, parts - 1, p =>
            {
                for (int i = l[p]; i < l[p + 1]; i++)
                    result[i] = MandelbrotEquation.Solve(x[i], y[i], maxit);
            });
            return result;
        }
    }
}
