#if NET35
// .NET 3.5 - Referencing System.Threading (Parallel extensions)
using System.Threading;
#else
// .NET 4.0
using System.Threading.Tasks;
using System.Collections.Concurrent;
#endif

using System.Diagnostics;

namespace MandelbrotDemo
{
    /// <summary>
    /// The SolveFrame contains the coordinates of the image and the resulting ImageBuffer
    /// </summary>
    public class SolverFrame
    {
        // Frame coordinates : Rectangle(X0,Y0,X1,Y1) 
        public double X0 { get; set; }
        public double Y0 { get; set; }
        public double X1 { get; set; }
        public double Y1 { get; set; }

        // Number of pixels
        public int Width { get; set; }
        public int Height { get; set; }

        // Image buffer
        public int[] ImageBuffer { get; private set; }
        private double[] x;
        private double[] y;

        public int MaxIterations { get; set; }

        public IEquationSolver Solver { get; set; }

        public SolverFrame()
        {
            X0 = -2; Y0 = -1.4;
            X1 = 1; Y1 = 1.4;
        }

        public void Update()
        {
            var w = new Stopwatch();
            w.Start();
            Initialize();
            long timeInit = w.ElapsedTicks;
            w.Reset();
            w.Start();
            ImageBuffer = Solver.Solve(x, y, MaxIterations);
            long timeSolve = w.ElapsedTicks;
            // Trace.WriteLine(t1 + "/" + t2 + "=" + (double)t1 / t2);
        }

        /// <summary>
        /// Initialize the x/y arrays (using a Parallel loop)
        /// </summary>
        private void Initialize()
        {
            x = new double[Width * Height];
            y = new double[Width * Height];

            double dx = (X1 - X0) / (Width - 1);
            double dy = (Y1 - Y0) / (Height - 1);
            
            double x0 = X0;
            double y0 = Y0;
            int w = Width;

#if ParallelInitialization
            Parallel.For(0, Height, j =>
#else
            for (int j=0;j<Height;j++) 
#endif
            {
                int ij = j * w;
                for (int i = 0; i < w; i++)
                {
                    x[ij] = x0 + dx * i;
                    y[ij++] = y0 + dy * j;
                }
            }
#if ParallelInitialization
            );
#endif
        }

        public void Zoom(double u, double v, double f)
        {
            double x = X0 + (X1 - X0) * u;
            double y = Y0 + (Y1 - Y0) * v;
            X0 += (x - X0) * f;
            X1 += (x - X1) * f;
            Y0 += (y - Y0) * f;
            Y1 += (y - Y1) * f;
        }

        public void Pan(double du, double dv)
        {
            double w = X1 - X0;
            double h = Y1 - Y0;
            X0 += w * du;
            X1 += w * du;
            Y0 += h * dv;
            Y1 += h * dv;
        }


    }
}